const empty_http = [];
const empty_urlencoded_http = [];
const empty_multipart_http = [];
const empty_multipart2_http = [];
const _minimal_http = [];

export {
  empty_http,
  empty_urlencoded_http,
  empty_multipart_http,
  empty_multipart2_http,
  _minimal_http,
};
