const menu_separator_png_http = [
  {
    type: 'file',
    name: 'image',
    originalFilename: 'menu_separator.png',
    fixture: 'menu_separator.png',
    sha1: 'c845ca3ea794be298f2a1b79769b71939eaf4e54',
  },
];

const beta_sticker_1_png_http = [
  {
    type: 'file',
    name: 'sticker',
    originalFilename: 'beta-sticker-1.png',
    fixture: 'beta-sticker-1.png',
    sha1: '6abbcffd12b4ada5a6a084fe9e4584f846331bc4',
  },
];

const blank_gif_http = [
  {
    type: 'file',
    name: 'file',
    originalFilename: 'blank.gif',
    fixture: 'blank.gif',
    sha1: 'a1fdee122b95748d81cee426d717c05b5174fe96',
  },
];

const binaryfile_tar_gz_http = [
  {
    type: 'file',
    name: 'file',
    originalFilename: 'binaryfile.tar.gz',
    fixture: 'binaryfile.tar.gz',
    sha1: 'cfabe13b348e5e69287d677860880c52a69d2155',
  },
];

const plain_txt_http = [
  {
    type: 'file',
    name: 'file',
    originalFilename: 'plain.txt',
    fixture: 'plain.txt',
    sha1: 'b31d07bac24ac32734de88b3687dddb10e976872',
  },
];

export {
  menu_separator_png_http,
  beta_sticker_1_png_http,
  blank_gif_http,
  binaryfile_tar_gz_http,
  plain_txt_http,
};
