const generic_http = [
  {
    type: 'file',
    name: 'upload',
    originalFilename: '',
    fixture: 'generic',
    sha1: 'b31d07bac24ac32734de88b3687dddb10e976872',
  },
];

const filename_name_http = [
  {
    type: 'file',
    name: 'upload',
    originalFilename: 'plain.txt',
    fixture: 'filename-name',
    sha1: 'a47f7a8a7959f36c3f151ba8b0bd28f2d6b606e2',
  },
];

export { generic_http, filename_name_http };
