const missing_hyphens1_http = [
  {
    type: 'file',
    name: 'upload',
    originalFilename: 'plain.txt',
    fixture: 'missing-hyphens1',
    sha1: '8c26b82ec9107e99b3486844644e92558efe0c73',
  },
];

const missing_hyphens2_http = [
  {
    type: 'file',
    name: 'upload',
    originalFilename: 'second-plaintext.txt',
    fixture: 'missing-hyphens2',
    sha1: '798e39a4a1034232ed26e0aadd67f5d1ff10b966',
  },
];

export { missing_hyphens1_http, missing_hyphens2_http };
